/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2009 The LS-Universe Team

  Homepage: http://www.ls-universe.info

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "PaintTexture.h"

#include "SettingsHelpers.h"
#include "PaintHelpers.h"

#include "ExtractResource.h"
#include "shellapi.h"

// XP Themes
typedef BOOL(__stdcall *PFNISTHEMEACTIVE)(VOID);
typedef HTHEME(__stdcall *PFNOPENTHEMEDATA)(HWND hwnd, LPCWSTR pszClassList);
typedef HRESULT(__stdcall *PFNCLOSETHEMEDATA)(HTHEME hTheme);
typedef HRESULT(__stdcall *PFNDRAWTHEMEBACKGROUND)(HTHEME hTheme, HDC hdc, 
                            int iPartId, int iStateId, const RECT *pRect,  const RECT *pClipRect);
typedef HRESULT(__stdcall *PFNGETTHEMEPARTSIZE)(HTHEME hTheme, HDC hdc,
                            int iPartId, int iStateId, LPCRECT prc, THEMESIZE eSize, SIZE *psz);                           
//typedef BOOL (__stdcall *PFNISTHEMEPARTDEFINED)(HTHEME hTheme, 
//                          int iPartId, int iStateId);
//typedef HRESULT (__stdcall *PFNGETTHEMEBACKGROUNDEXTENT)(HTHEME hTheme,  HDC hdc,
//      int iPartId, int iStateId, const RECT *pContentRect, 
//      RECT *pExtentRect);

static PFNISTHEMEACTIVE zIsThemeActive = NULL;
static PFNOPENTHEMEDATA zOpenThemeData = NULL;
static PFNCLOSETHEMEDATA zCloseThemeData = NULL;
static PFNDRAWTHEMEBACKGROUND zDrawThemeBackground = NULL;
static PFNGETTHEMEPARTSIZE zGetThemePartSize = NULL;
//static PFNISTHEMEPARTDEFINED zIsThemePartDefined = NULL;
//static PFNGETTHEMEBACKGROUNDEXTENT zGetThemeBackgroundExtent = NULL;

static HMODULE hModThemes = NULL;

bool InitThemes()
{
    if ( !hModThemes )
    {
        hModThemes = LoadLibrary("UXTHEME.DLL");

        if ( hModThemes )
        {
            zIsThemeActive = (PFNISTHEMEACTIVE)GetProcAddress(hModThemes, "IsThemeActive");
            zOpenThemeData = (PFNOPENTHEMEDATA)GetProcAddress(hModThemes, "OpenThemeData");
            zCloseThemeData = (PFNCLOSETHEMEDATA)GetProcAddress(hModThemes, "CloseThemeData");
            zDrawThemeBackground = (PFNDRAWTHEMEBACKGROUND)GetProcAddress(hModThemes, "DrawThemeBackground");
            zGetThemePartSize = (PFNGETTHEMEPARTSIZE)GetProcAddress(hModThemes, "GetThemePartSize");

            //zIsThemePartDefined = (PFNISTHEMEPARTDEFINED)GetProcAddress(hModThemes, "IsThemePartDefined");
            //zGetThemeBackgroundExtent = (PFNGETTHEMEBACKGROUNDEXTENT)GetProcAddress(hModThemes, "GetThemeBackgroundExtent");
            
            if ( zIsThemeActive && zOpenThemeData && zDrawThemeBackground && zGetThemePartSize /*&& zIsThemePartDefined && zGetThemeBackgroundExtent */&& zCloseThemeData )
            {
                return true;            
            }
            else
            {
                FreeLibrary(hModThemes);
                hModThemes = NULL;

                return false;   
            }
        }
        
        return false;   
    }
    else
    {
        return true;
    }
}

HRESULT Create_hThemeImage(HTHEME hTheme, HDC destDC, int iPart, int iState, SIZE sInput, RECT rOutput)
{
    int iInputWidth = sInput.cx;
    int iInputHeight = sInput.cy;

    RECT rInput;
    rInput.left = 0;
    rInput.right = iInputWidth;
    rInput.top = 0;
    rInput.bottom = iInputHeight;

    HDC bufferhThemeDC = CreateCompatibleDC(NULL);

    BITMAPINFO alpha;
    ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
    alpha.bmiHeader.biWidth=iInputWidth;      // Set size you need
    alpha.bmiHeader.biHeight=iInputHeight;    // Set size you need
    alpha.bmiHeader.biPlanes=1;
    alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
    alpha.bmiHeader.biSizeImage=0;
    alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
    alpha.bmiHeader.biClrUsed= 0;
    alpha.bmiHeader.biClrImportant= 0;
    VOID *pvBits;

    HBITMAP bufferhThemeBitmap = CreateDIBSection( bufferhThemeDC,
                                                    &alpha,
                                                    DIB_RGB_COLORS,
                                                    &pvBits,
                                                    NULL,
                                                    0 );

    HBITMAP oldhThemeBMP = (HBITMAP) SelectObject(bufferhThemeDC, bufferhThemeBitmap);

    SetStretchBltMode(bufferhThemeDC, STRETCH_DELETESCANS);

    HRESULT hr = zDrawThemeBackground(hTheme, bufferhThemeDC, iPart, iState, &rInput, NULL);
                
    if ( hr == S_OK )
    {
        MultiBlt(destDC, 0, 0, rOutput.right, rOutput.bottom,
                            bufferhThemeDC, 0, 0, iInputWidth, iInputHeight,
                            0, 0, 0, 0,
                            MULTIBLT_STRETCH_QUADRATIC, SRCCOPY);
    }

    bufferhThemeBitmap = (HBITMAP) SelectObject(bufferhThemeDC, oldhThemeBMP);
    DeleteDC(bufferhThemeDC);   
    DeleteObject(bufferhThemeBitmap);
    DeleteObject(oldhThemeBMP);
    
    return hr;
}

PaintTexture::PaintTexture() :
    m_bSpecialSetup(false),

    m_iOldPaintingWidth(-1),
    m_iOldPaintingHeight(-1),
    
    m_bUseAlphaMap(false),
    
    memoryDC(NULL),
    memoryBitmap(NULL), 

    m_strTexture_X("0"),
    m_strTexture_Y("0"),

    m_strTexture_Width("100%"),
    m_strTexture_Height("100%"),

    m_iPaintingMode(-1),

    //-----------------------------------------------------------------
    //Colors or Image
    //-----------------------------------------------------------------

    m_bDisablePerformanceCaching(false),

    m_iBorderMethod(0),

    m_iAlphaTransparency(255),

    m_bTrueTransparency(false),

    m_strImagePath(""),
    m_strIconPath(""),

    //-----------------------------------------------------------------
    //Icon
    //-----------------------------------------------------------------

    m_iExtractIconSize(32),

    //-----------------------------------------------------------------
    //Image/Icon
    //-----------------------------------------------------------------
    
    m_iImageHeight(64),
    m_iImageWidth(64),  
    
    m_strImageEdges(""),
    m_iLeftImageEdge(0),
    m_iTopImageEdge(0),
    m_iRightImageEdge(0),
    m_iBottomImageEdge(0),

    m_strImageCrop(""),
    m_iLeftImageCrop(0),
    m_iTopImageCrop(0),
    m_iRightImageCrop(0),
    m_iBottomImageCrop(0),

    m_iMode(MULTIBLT_STRETCH),

    m_clrHueColor(RGB(255,255,255)),
    m_iHueIntensity(0),
    m_iLuminanceIntensity(0),   
    m_iSaturationIntensity(100),

    m_clrMixColor(RGB(255,255,255)),
    m_iMixIntensity(0),

    m_iFrameWidth(0),
    m_iFrameHeight(0),

    //-----------------------------------------------------------------
    //Colors
    //-----------------------------------------------------------------

    m_strColors(""),

    m_clrBackgroundColor(RGB(255,255,255)),
    m_clrDarkBevelColor(GetSysColor(COLOR_3DLIGHT)),
    m_clrLightBevelColor(GetSysColor(COLOR_3DSHADOW)),

    m_strGradientColors(""),

    m_strGradientRepeatedColors(""),

    m_iGradientType(0),
    m_iGradientTransformationType(-1),

    m_strBevels(""),
    m_iLeftBevel(0),
    m_iRightBevel(0),
    m_iTopBevel(0),
    m_iBottomBevel(0),

    //Update 2
    m_iShapeMode(1),
    m_strRoundedEdges("0"),
    m_iRounded_X(0),
    m_iRounded_Y(0),

    //Update 4
    m_strTexture_Rotation("0"),
    m_iTexture_Rotation(0),
    m_iRotationWidth(-1),
    m_iRotationHeight(-1),

    //Update 5
    m_strInternalMSStyle(""),
    m_strMSStyle_Class(L""),
    m_iMSStyle_Parts(0),
    m_iMSStyle_States(0),

    //Update 6
    m_iIconMode(0),

	//Update 7
	m_bAlphaTranslucency(false)
{
    GradientColors.clear();
    GradientRepeatedColors.clear();
}

PaintTexture::PaintTexture(const PaintTextureDefaults *pPaintTextureDefaults) :
    m_bSpecialSetup(false),

    m_iOldPaintingWidth(-1),
    m_iOldPaintingHeight(-1),
    
    m_bUseAlphaMap(false),

    memoryDC(NULL),
    memoryBitmap(NULL)
{
    m_strTexture_X = pPaintTextureDefaults->m_strTexture_X;
    m_strTexture_Y = pPaintTextureDefaults->m_strTexture_Y;

    m_strTexture_Width = pPaintTextureDefaults->m_strTexture_Width;
    m_strTexture_Height = pPaintTextureDefaults->m_strTexture_Height;

    m_iPaintingMode = pPaintTextureDefaults->m_iPaintingMode;

    //-----------------------------------------------------------------
    //Colors or Image
    //-----------------------------------------------------------------

    m_bDisablePerformanceCaching = (pPaintTextureDefaults->m_bDisablePerformanceCaching ? true : false);

    m_iBorderMethod = pPaintTextureDefaults->m_iBorderMethod;

    m_iAlphaTransparency = pPaintTextureDefaults->m_iAlphaTransparency;

    m_bTrueTransparency = (pPaintTextureDefaults->m_bTrueTransparency ? true : false);

    m_strImagePath = pPaintTextureDefaults->m_strImagePath;
    m_strIconPath = pPaintTextureDefaults->m_strIconPath;

    //-----------------------------------------------------------------
    //Icon
    //-----------------------------------------------------------------

    m_iExtractIconSize = pPaintTextureDefaults->m_iExtractIconSize;

    //-----------------------------------------------------------------
    //Image/Icon
    //-----------------------------------------------------------------
    
    m_iImageHeight = pPaintTextureDefaults->m_iImageHeight;
    m_iImageWidth = pPaintTextureDefaults->m_iImageWidth;
    
    m_strImageEdges = pPaintTextureDefaults->m_strImageEdges;
    m_iLeftImageEdge = pPaintTextureDefaults->m_iLeftImageEdge;
    m_iTopImageEdge = pPaintTextureDefaults->m_iTopImageEdge;
    m_iRightImageEdge = pPaintTextureDefaults->m_iRightImageEdge;
    m_iBottomImageEdge = pPaintTextureDefaults->m_iBottomImageEdge;

    m_strImageCrop = pPaintTextureDefaults->m_strImageCrop;
    m_iLeftImageCrop = pPaintTextureDefaults->m_iLeftImageCrop;
    m_iTopImageCrop = pPaintTextureDefaults->m_iTopImageCrop;
    m_iRightImageCrop = pPaintTextureDefaults->m_iRightImageCrop;
    m_iBottomImageCrop = pPaintTextureDefaults->m_iBottomImageCrop;

    m_iMode = pPaintTextureDefaults->m_iMode;

    m_clrHueColor = pPaintTextureDefaults->m_clrHueColor;
    m_iHueIntensity = pPaintTextureDefaults->m_iHueIntensity;
    m_iLuminanceIntensity = pPaintTextureDefaults->m_iLuminanceIntensity;
    m_iSaturationIntensity = pPaintTextureDefaults->m_iSaturationIntensity;

    m_clrMixColor = pPaintTextureDefaults->m_clrMixColor;
    m_iMixIntensity = pPaintTextureDefaults->m_iMixIntensity;

    m_iFrameWidth = pPaintTextureDefaults->m_iFrameWidth;
    m_iFrameHeight = pPaintTextureDefaults->m_iFrameHeight;

    //-----------------------------------------------------------------
    //Colors
    //-----------------------------------------------------------------

    m_strColors = pPaintTextureDefaults->m_strColors;

    m_clrBackgroundColor = pPaintTextureDefaults->m_clrBackgroundColor;
    m_clrDarkBevelColor = pPaintTextureDefaults->m_clrDarkBevelColor;
    m_clrLightBevelColor = pPaintTextureDefaults->m_clrLightBevelColor;

    m_strGradientColors = pPaintTextureDefaults->m_strGradientColors;

    m_strGradientRepeatedColors = pPaintTextureDefaults->m_strGradientRepeatedColors;

    m_iGradientType = pPaintTextureDefaults->m_iGradientType;
    m_iGradientTransformationType = pPaintTextureDefaults->m_iGradientTransformationType;

    m_strBevels = pPaintTextureDefaults->m_strBevels;
    m_iLeftBevel = pPaintTextureDefaults->m_iLeftBevel;
    m_iRightBevel = pPaintTextureDefaults->m_iRightBevel;
    m_iTopBevel = pPaintTextureDefaults->m_iTopBevel;
    m_iBottomBevel = pPaintTextureDefaults->m_iBottomBevel;

    GradientColors.clear();
    GradientRepeatedColors.clear();

    //Update 2
    //Cannot enable due to backwards compatibility!
    m_iShapeMode = 1;               //pPaintTextureDefaults->m_iShapeMode;
    m_strRoundedEdges = "0";        //pPaintTextureDefaults->m_strRoundedEdges;
    m_iRounded_X = 0;
    m_iRounded_Y = 0;

    //Update 4
    //Cannot enable due to backwards compatibility!
    m_strTexture_Rotation = "0";    //pPaintTextureDefaults->m_strTexture_Rotation;
    m_iTexture_Rotation = 0;
    m_iRotationWidth = -1;
    m_iRotationHeight = -1;

    //Update 5
    //Cannot enable due to backwards compatibility!
    m_strInternalMSStyle = "";      //pPaintTextureDefaults->m_strInternalMSStyle;
    m_strMSStyle_Class = L"";           
    m_iMSStyle_Parts = 0;           
    m_iMSStyle_States = 0;      

    //Update 6
    //Cannot enable due to backwards compatibility!
    m_iIconMode = 0;                //pPaintTextureDefaults->m_iIconMode;

	//Update 7
	//Cannot enable due to backwards compatibility!
	m_bAlphaTranslucency = false;   //pPaintTextureDefaults->m_bAlphaTranslucency;
}

PaintTexture::~PaintTexture()
{
    if ( hModThemes )
    {
        FreeLibrary(hModThemes);
        hModThemes = NULL;
    }

    GradientColors.clear();
    GradientRepeatedColors.clear();

    if(memoryBitmap)
    {
        memoryBitmap = (HBITMAP) SelectObject(memoryDC, memoryBitmap);
        DeleteObject(memoryBitmap);
    }
    if(memoryDC)
        DeleteDC(memoryDC);

    memoryBitmap = NULL;
    memoryDC = NULL;
}

BOOL PaintTexture::configure(LPCSTR pszName, LPCSTR pszPrefix, BOOL bUseAlphaMap, const PaintTexture *pDefaultPaintTexture, const PaintTextureDefaults *pPaintTextureDefaults)
{
    string name(pszName);
    string prefix(pszPrefix);

    string strAddToGroup = iGetxRCString(name, (prefix + "TextureAddToGroup"), "");

    if (!strAddToGroup.empty() && _stricmp(name.c_str(), strAddToGroup.c_str()) != NULL)
    {   
        return configure(strAddToGroup.c_str(), "", bUseAlphaMap, pDefaultPaintTexture, pPaintTextureDefaults);
    }

    bool bDifferenceToDefault = false;

    const NameValuePair paintingmodeConstants[] =
    {
        { ".none", -1 },
        { ".image", 0 },
        { ".icon", 1 },
        { ".multicolor", 2 },
        { ".singlecolor", 3 },
        { ".borderbutton", 4 },
        { ".animation", 5 },
        { ".visualstyle", 6 },
        { "none", -1 },
        { "image", 0 },
        { "icon", 1 },
        { "multicolor", 2 },
        { "singlecolor", 3 },
        { "borderbutton", 4 },
        { "animation", 5 },
        { "visualstyle", 6 },
        { 0, 0 }
    };

    const NameValuePair borderValues[] = 
    {
        { ".none", 0 },
        { ".raised", 1 },
        { ".sunken", 2 },
        { ".etched", 3 },
        { ".bump", 4 },
        { "none", 0 },
        { "raised", 1 },
        { "sunken", 2 },
        { "etched", 3 },
        { "bump", 4 },
        { 0, 0 }
    };

    const NameValuePair modeConstants[] =
    {
        { ".stretch", MULTIBLT_STRETCH },
        { ".tile", MULTIBLT_TILE },
        { ".tile-horizontal", MULTIBLT_TILEHORIZONTAL },
        { ".tile-vertical", MULTIBLT_TILEVERTICAL },
        { ".tilehorizontal", MULTIBLT_TILEHORIZONTAL },
        { ".tilevertical", MULTIBLT_TILEVERTICAL },

        //Update 6
        { ".stretch_bell", MULTIBLT_STRETCH_BELL },
        { ".stretch_box", MULTIBLT_STRETCH_BOX },
        { ".stretch_catmullrom", MULTIBLT_STRETCH_CATMULLROM },
        { ".stretch_cosine", MULTIBLT_STRETCH_COSINE },
        { ".stretch_cubicconvolution", MULTIBLT_STRETCH_CUBICCONVOLUTION },
        { ".stretch_cubicspline", MULTIBLT_STRETCH_CUBICSPLINE },
        { ".stretch_hermite", MULTIBLT_STRETCH_HERMITE },
        { ".stretch_lanczos3", MULTIBLT_STRETCH_LANCZOS3 },
        { ".stretch_lanczos8", MULTIBLT_STRETCH_LANCZOS8 },
        { ".stretch_mitchell", MULTIBLT_STRETCH_MITCHELL },
        { ".stretch_quadratic", MULTIBLT_STRETCH_QUADRATIC },
        { ".stretch_quadraticbspline", MULTIBLT_STRETCH_QUADRATICBSPLINE },
        { ".stretch_triangle", MULTIBLT_STRETCH_TRIANGLE },

        { ".tile-vertical_bell", MULTIBLT_TILEVERTICAL_BELL },
        { ".tile-vertical_box", MULTIBLT_TILEVERTICAL_BOX },
        { ".tile-vertical_catmullrom", MULTIBLT_TILEVERTICAL_CATMULLROM },
        { ".tile-vertical_cosine", MULTIBLT_TILEVERTICAL_COSINE },
        { ".tile-vertical_cubicconvolution", MULTIBLT_TILEVERTICAL_CUBICCONVOLUTION },
        { ".tile-vertical_cubicspline", MULTIBLT_TILEVERTICAL_CUBICSPLINE },
        { ".tile-vertical_hermite", MULTIBLT_TILEVERTICAL_HERMITE },
        { ".tile-vertical_lanczos3", MULTIBLT_TILEVERTICAL_LANCZOS3 },
        { ".tile-vertical_lanczos8", MULTIBLT_TILEVERTICAL_LANCZOS8 },
        { ".tile-vertical_mitchell", MULTIBLT_TILEVERTICAL_MITCHELL },
        { ".tile-vertical_quadratic", MULTIBLT_TILEVERTICAL_QUADRATIC },
        { ".tile-vertical_quadraticbspline", MULTIBLT_TILEVERTICAL_QUADRATICBSPLINE },
        { ".tile-vertical_triangle", MULTIBLT_TILEVERTICAL_TRIANGLE },
        { ".tilevertical_bell", MULTIBLT_TILEVERTICAL_BELL },
        { ".tilevertical_box", MULTIBLT_TILEVERTICAL_BOX },
        { ".tilevertical_catmullrom", MULTIBLT_TILEVERTICAL_CATMULLROM },
        { ".tilevertical_cosine", MULTIBLT_TILEVERTICAL_COSINE },
        { ".tilevertical_cubicconvolution", MULTIBLT_TILEVERTICAL_CUBICCONVOLUTION },
        { ".tilevertical_cubicspline", MULTIBLT_TILEVERTICAL_CUBICSPLINE },
        { ".tilevertical_hermite", MULTIBLT_TILEVERTICAL_HERMITE },
        { ".tilevertical_lanczos3", MULTIBLT_TILEVERTICAL_LANCZOS3 },
        { ".tilevertical_lanczos8", MULTIBLT_TILEVERTICAL_LANCZOS8 },
        { ".tilevertical_mitchell", MULTIBLT_TILEVERTICAL_MITCHELL },
        { ".tilevertical_quadratic", MULTIBLT_TILEVERTICAL_QUADRATIC },
        { ".tilevertical_quadraticbspline", MULTIBLT_TILEVERTICAL_QUADRATICBSPLINE },
        { ".tilevertical_triangle", MULTIBLT_TILEVERTICAL_TRIANGLE },
        
        { ".tile-horizontal_bell", MULTIBLT_TILEHORIZONTAL_BELL },
        { ".tile-horizontal_box", MULTIBLT_TILEHORIZONTAL_BOX },
        { ".tile-horizontal_catmullrom", MULTIBLT_TILEHORIZONTAL_CATMULLROM },
        { ".tile-horizontal_cosine", MULTIBLT_TILEHORIZONTAL_COSINE },
        { ".tile-horizontal_cubicconvolution", MULTIBLT_TILEHORIZONTAL_CUBICCONVOLUTION },
        { ".tile-horizontal_cubicspline", MULTIBLT_TILEHORIZONTAL_CUBICSPLINE },
        { ".tile-horizontal_hermite", MULTIBLT_TILEHORIZONTAL_HERMITE },
        { ".tile-horizontal_lanczos3", MULTIBLT_TILEHORIZONTAL_LANCZOS3 },
        { ".tile-horizontal_lanczos8", MULTIBLT_TILEHORIZONTAL_LANCZOS8 },
        { ".tile-horizontal_mitchell", MULTIBLT_TILEHORIZONTAL_MITCHELL },
        { ".tile-horizontal_quadratic", MULTIBLT_TILEHORIZONTAL_QUADRATIC },
        { ".tile-horizontal_quadraticbspline", MULTIBLT_TILEHORIZONTAL_QUADRATICBSPLINE },
        { ".tile-horizontal_triangle", MULTIBLT_TILEHORIZONTAL_TRIANGLE },
        { ".tilehorizontal_bell", MULTIBLT_TILEHORIZONTAL_BELL },
        { ".tilehorizontal_box", MULTIBLT_TILEHORIZONTAL_BOX },
        { ".tilehorizontal_catmullrom", MULTIBLT_TILEHORIZONTAL_CATMULLROM },
        { ".tilehorizontal_cosine", MULTIBLT_TILEHORIZONTAL_COSINE },
        { ".tilehorizontal_cubicconvolution", MULTIBLT_TILEHORIZONTAL_CUBICCONVOLUTION },
        { ".tilehorizontal_cubicspline", MULTIBLT_TILEHORIZONTAL_CUBICSPLINE },
        { ".tilehorizontal_hermite", MULTIBLT_TILEHORIZONTAL_HERMITE },
        { ".tilehorizontal_lanczos3", MULTIBLT_TILEHORIZONTAL_LANCZOS3 },
        { ".tilehorizontal_lanczos8", MULTIBLT_TILEHORIZONTAL_LANCZOS8 },
        { ".tilehorizontal_mitchell", MULTIBLT_TILEHORIZONTAL_MITCHELL },
        { ".tilehorizontal_quadratic", MULTIBLT_TILEHORIZONTAL_QUADRATIC },
        { ".tilehorizontal_quadraticbspline", MULTIBLT_TILEHORIZONTAL_QUADRATICBSPLINE },
        { ".tilehorizontal_triangle", MULTIBLT_TILEHORIZONTAL_TRIANGLE },
        //Update 6

        { "stretch", MULTIBLT_STRETCH },
        { "tile", MULTIBLT_TILE },
        { "tile-horizontal", MULTIBLT_TILEHORIZONTAL },
        { "tile-vertical", MULTIBLT_TILEVERTICAL },
        { "tilehorizontal", MULTIBLT_TILEHORIZONTAL },
        { "tilevertical", MULTIBLT_TILEVERTICAL },

        //Update 6
        { "stretch_bell", MULTIBLT_STRETCH_BELL },
        { "stretch_box", MULTIBLT_STRETCH_BOX },
        { "stretch_catmullrom", MULTIBLT_STRETCH_CATMULLROM },
        { "stretch_cosine", MULTIBLT_STRETCH_COSINE },
        { "stretch_cubicconvolution", MULTIBLT_STRETCH_CUBICCONVOLUTION },
        { "stretch_cubicspline", MULTIBLT_STRETCH_CUBICSPLINE },
        { "stretch_hermite", MULTIBLT_STRETCH_HERMITE },
        { "stretch_lanczos3", MULTIBLT_STRETCH_LANCZOS3 },
        { "stretch_lanczos8", MULTIBLT_STRETCH_LANCZOS8 },
        { "stretch_mitchell", MULTIBLT_STRETCH_MITCHELL },
        { "stretch_quadratic", MULTIBLT_STRETCH_QUADRATIC },
        { "stretch_quadraticbspline", MULTIBLT_STRETCH_QUADRATICBSPLINE },
        { "stretch_triangle", MULTIBLT_STRETCH_TRIANGLE },

        { "tile-vertical_bell", MULTIBLT_TILEVERTICAL_BELL },
        { "tile-vertical_box", MULTIBLT_TILEVERTICAL_BOX },
        { "tile-vertical_catmullrom", MULTIBLT_TILEVERTICAL_CATMULLROM },
        { "tile-vertical_cosine", MULTIBLT_TILEVERTICAL_COSINE },
        { "tile-vertical_cubicconvolution", MULTIBLT_TILEVERTICAL_CUBICCONVOLUTION },
        { "tile-vertical_cubicspline", MULTIBLT_TILEVERTICAL_CUBICSPLINE },
        { "tile-vertical_hermite", MULTIBLT_TILEVERTICAL_HERMITE },
        { "tile-vertical_lanczos3", MULTIBLT_TILEVERTICAL_LANCZOS3 },
        { "tile-vertical_lanczos8", MULTIBLT_TILEVERTICAL_LANCZOS8 },
        { "tile-vertical_mitchell", MULTIBLT_TILEVERTICAL_MITCHELL },
        { "tile-vertical_quadratic", MULTIBLT_TILEVERTICAL_QUADRATIC },
        { "tile-vertical_quadraticbspline", MULTIBLT_TILEVERTICAL_QUADRATICBSPLINE },
        { "tile-vertical_triangle", MULTIBLT_TILEVERTICAL_TRIANGLE },
        { "tilevertical_bell", MULTIBLT_TILEVERTICAL_BELL },
        { "tilevertical_box", MULTIBLT_TILEVERTICAL_BOX },
        { "tilevertical_catmullrom", MULTIBLT_TILEVERTICAL_CATMULLROM },
        { "tilevertical_cosine", MULTIBLT_TILEVERTICAL_COSINE },
        { "tilevertical_cubicconvolution", MULTIBLT_TILEVERTICAL_CUBICCONVOLUTION },
        { "tilevertical_cubicspline", MULTIBLT_TILEVERTICAL_CUBICSPLINE },
        { "tilevertical_hermite", MULTIBLT_TILEVERTICAL_HERMITE },
        { "tilevertical_lanczos3", MULTIBLT_TILEVERTICAL_LANCZOS3 },
        { "tilevertical_lanczos8", MULTIBLT_TILEVERTICAL_LANCZOS8 },
        { "tilevertical_mitchell", MULTIBLT_TILEVERTICAL_MITCHELL },
        { "tilevertical_quadratic", MULTIBLT_TILEVERTICAL_QUADRATIC },
        { "tilevertical_quadraticbspline", MULTIBLT_TILEVERTICAL_QUADRATICBSPLINE },
        { "tilevertical_triangle", MULTIBLT_TILEVERTICAL_TRIANGLE },
        
        { "tile-horizontal_bell", MULTIBLT_TILEHORIZONTAL_BELL },
        { "tile-horizontal_box", MULTIBLT_TILEHORIZONTAL_BOX },
        { "tile-horizontal_catmullrom", MULTIBLT_TILEHORIZONTAL_CATMULLROM },
        { "tile-horizontal_cosine", MULTIBLT_TILEHORIZONTAL_COSINE },
        { "tile-horizontal_cubicconvolution", MULTIBLT_TILEHORIZONTAL_CUBICCONVOLUTION },
        { "tile-horizontal_cubicspline", MULTIBLT_TILEHORIZONTAL_CUBICSPLINE },
        { "tile-horizontal_hermite", MULTIBLT_TILEHORIZONTAL_HERMITE },
        { "tile-horizontal_lanczos3", MULTIBLT_TILEHORIZONTAL_LANCZOS3 },
        { "tile-horizontal_lanczos8", MULTIBLT_TILEHORIZONTAL_LANCZOS8 },
        { "tile-horizontal_mitchell", MULTIBLT_TILEHORIZONTAL_MITCHELL },
        { "tile-horizontal_quadratic", MULTIBLT_TILEHORIZONTAL_QUADRATIC },
        { "tile-horizontal_quadraticbspline", MULTIBLT_TILEHORIZONTAL_QUADRATICBSPLINE },
        { "tile-horizontal_triangle", MULTIBLT_TILEHORIZONTAL_TRIANGLE },
        { "tilehorizontal_bell", MULTIBLT_TILEHORIZONTAL_BELL },
        { "tilehorizontal_box", MULTIBLT_TILEHORIZONTAL_BOX },
        { "tilehorizontal_catmullrom", MULTIBLT_TILEHORIZONTAL_CATMULLROM },
        { "tilehorizontal_cosine", MULTIBLT_TILEHORIZONTAL_COSINE },
        { "tilehorizontal_cubicconvolution", MULTIBLT_TILEHORIZONTAL_CUBICCONVOLUTION },
        { "tilehorizontal_cubicspline", MULTIBLT_TILEHORIZONTAL_CUBICSPLINE },
        { "tilehorizontal_hermite", MULTIBLT_TILEHORIZONTAL_HERMITE },
        { "tilehorizontal_lanczos3", MULTIBLT_TILEHORIZONTAL_LANCZOS3 },
        { "tilehorizontal_lanczos8", MULTIBLT_TILEHORIZONTAL_LANCZOS8 },
        { "tilehorizontal_mitchell", MULTIBLT_TILEHORIZONTAL_MITCHELL },
        { "tilehorizontal_quadratic", MULTIBLT_TILEHORIZONTAL_QUADRATIC },
        { "tilehorizontal_quadraticbspline", MULTIBLT_TILEHORIZONTAL_QUADRATICBSPLINE },
        { "tilehorizontal_triangle", MULTIBLT_TILEHORIZONTAL_TRIANGLE },
        //Update 6
        { 0, 0 }
    };

    const NameValuePair gradientmodeConstants[] =
    {
        { ".none", GRADIENT_NONE },
        { ".horizontal", GRADIENT_HORIZONTAL },
        { ".vertical", GRADIENT_VERTICAL },
        { ".radial", GRADIENT_RADIAL },
        { ".diagonal", GRADIENT_DIAGONAL },
        { ".fdiagonal", GRADIENT_FDIAGONAL },
        { ".bdiagonal", GRADIENT_BDIAGONAL },
        //Update 6 by Tobbe
        { ".pipecross", GRADIENT_PIPECROSS },
        { ".elliptic", GRADIENT_ELLIPTIC },
        { ".rectangle", GRADIENT_RECTANGLE },
        { ".pyramid", GRADIENT_PYRAMID },
        //Update
        { "none", GRADIENT_NONE },
        { "horizontal", GRADIENT_HORIZONTAL },
        { "vertical", GRADIENT_VERTICAL },
        { "radial", GRADIENT_RADIAL },
        { "diagonal", GRADIENT_DIAGONAL },
        { "fdiagonal", GRADIENT_FDIAGONAL },
        { "bdiagonal", GRADIENT_BDIAGONAL },
        //Update 6 by Tobbe
        { "pipecross", GRADIENT_PIPECROSS },
        { "elliptic", GRADIENT_ELLIPTIC },
        { "rectangle", GRADIENT_RECTANGLE },
        { "pyramid", GRADIENT_PYRAMID },
        //Update
        { 0, 0 }
    };

    const NameValuePair gradienttransformationConstants[] =
    {
        { ".none", TRANSFORMATION_NONE },
        { ".caricature", TRANSFORMATION_CHARICATURE },
        { ".fisheye", TRANSFORMATION_FISHEYE },
        { ".swirled", TRANSFORMATION_SWIRLED },
        { ".cylinder", TRANSFORMATION_CYLINDER },
        { ".shift", TRANSFORMATION_SHIFT },
        { "none", TRANSFORMATION_NONE },
        { "caricature", TRANSFORMATION_CHARICATURE },
        { "fisheye", TRANSFORMATION_FISHEYE },
        { "swirled", TRANSFORMATION_SWIRLED },
        { "cylinder", TRANSFORMATION_CYLINDER },
        { "shift", TRANSFORMATION_SHIFT },
        { 0, 0 }
    };

    const NameValuePair shapemodeConstants[] =
    {
        { ".rectangle", 1 },
        { ".ellipse", 2 },
        { ".rounded", 3 },
        { "rectangle", 1 },
        { "ellipse", 2 },
        { "rounded", 3 },
        { 0, 0 }
    };

    //Reset All, Due to "reconfigure"
    //---------------------------------------------------------------------------------------
    GradientColors.clear();

    if(memoryBitmap)
    {
        memoryBitmap = (HBITMAP) SelectObject(memoryDC, memoryBitmap);
        DeleteObject(memoryBitmap);
    }
    if(memoryDC)
        DeleteDC(memoryDC);

    memoryBitmap = NULL;
    memoryDC = NULL;
    //---------------------------------------------------------------------------------------

    //Create Dummy Default
    bool bDefaultPaintTextureCreated = false;
    if ( pDefaultPaintTexture == NULL )
    {
        bDefaultPaintTextureCreated = true;
        if ( pPaintTextureDefaults == NULL )
            pDefaultPaintTexture = new PaintTexture();
        else
            pDefaultPaintTexture = new PaintTexture(pPaintTextureDefaults);
    }

    m_bUseAlphaMap = bUseAlphaMap;

    //-----------------------------------------------------------------
    m_strTexture_X = iGetx2RCString(name, (prefix + "TextureX"), pDefaultPaintTexture->m_strTexture_X);
    if ( _stricmp(m_strTexture_X.c_str(), "0") != NULL )
        m_bSpecialSetup = true;
    if ( _stricmp(m_strTexture_X.c_str(), pDefaultPaintTexture->m_strTexture_X.c_str()) != NULL )
        bDifferenceToDefault = true;
    
    m_strTexture_Y = iGetx2RCString(name, (prefix + "TextureY"), pDefaultPaintTexture->m_strTexture_Y);
    if ( _stricmp(m_strTexture_Y.c_str(), "0") != NULL )
        m_bSpecialSetup = true;
    if ( _stricmp(m_strTexture_Y.c_str(), pDefaultPaintTexture->m_strTexture_Y.c_str()) != NULL )
        bDifferenceToDefault = true;

    m_strTexture_Width = iGetx2RCString(name, (prefix + "TextureWidth"), pDefaultPaintTexture->m_strTexture_Width);
    if ( _stricmp(m_strTexture_Width.c_str(), "100%") != NULL )
        m_bSpecialSetup = true;
    if ( _stricmp(m_strTexture_Width.c_str(), pDefaultPaintTexture->m_strTexture_Width.c_str()) != NULL )
        bDifferenceToDefault = true;
    
    m_strTexture_Height = iGetx2RCString(name, (prefix + "TextureHeight"), pDefaultPaintTexture->m_strTexture_Height);
    if ( _stricmp(m_strTexture_Height.c_str(), "100%") != NULL )
        m_bSpecialSetup = true;
    if ( _stricmp(m_strTexture_Height.c_str(), pDefaultPaintTexture->m_strTexture_Height.c_str()) != NULL )
        bDifferenceToDefault = true;

    //-----------------------------------------------------------------

    //What Mode? -1 Deactivated, 0 Image, 1 Icon, 2 MultiColor, 3 SingleColor, 4 BorderButton, 6 MSStyle
    m_iPaintingMode = iGetx2RCNamedValue(name, (prefix + "PaintingMode"), paintingmodeConstants, pDefaultPaintTexture->m_iPaintingMode);
    
    //Security fallback if XP or newer isn't available 
    if ( m_iPaintingMode == 6 )
    {
        if ( !IsOS(OS_XP) )
        {
            MessageBox(NULL, (name + prefix + "PaintingMode: \".visualstyle\"\nYou need at least Windows XP to use the PaintingMode: \".visualstyle\"").c_str(), "Operating System Error", MB_OK | MB_ICONWARNING | MB_TOPMOST | MB_SETFOREGROUND);
            m_iPaintingMode = 2;
        }
        else
        {
            if ( InitThemes() )
            {
                if ( !zIsThemeActive() )
                {
                    MessageBox(NULL, (name + prefix + "PaintingMode: \".visualstyle\"\nVisual styles seem to be deactivated!").c_str(), "Visual Styles Error", MB_OK | MB_ICONWARNING | MB_TOPMOST | MB_SETFOREGROUND);
                    m_iPaintingMode = 2;
                }
            }
            else
            {
                MessageBox(NULL, (name + prefix + "PaintingMode: \".visualstyle\"\nDll initialization failed! Check your uxtheme.dll!").c_str(), "uxtheme.dll Error!", MB_OK | MB_ICONWARNING | MB_TOPMOST | MB_SETFOREGROUND);
                m_iPaintingMode = 2;
            }
        }
    }

    if ( m_iPaintingMode != pDefaultPaintTexture->m_iPaintingMode )
        bDifferenceToDefault = true;

    //-----------------------------------------------------------------
    //Get All Values
    //-----------------------------------------------------------------

    m_bDisablePerformanceCaching = iGetx2RCBool(name, (prefix + "DisablePerformanceCaching"), pDefaultPaintTexture->m_bDisablePerformanceCaching);
    if ( m_bDisablePerformanceCaching != pDefaultPaintTexture->m_bDisablePerformanceCaching )
        bDifferenceToDefault = true;

    m_iBorderMethod = iGetx2RCNamedValue(name, (prefix + "BorderMethod"), borderValues, pDefaultPaintTexture->m_iBorderMethod);
    if ( m_iBorderMethod != pDefaultPaintTexture->m_iBorderMethod )
        bDifferenceToDefault = true;

    m_iAlphaTransparency = iGetx2RCInt(name, (prefix + "TextureAlphaTransparency"), pDefaultPaintTexture->m_iAlphaTransparency, 0, 255);
    if ( m_iAlphaTransparency != pDefaultPaintTexture->m_iAlphaTransparency )
        bDifferenceToDefault = true;

    m_bTrueTransparency = iGetx2RCBool(name, (prefix + "TextureTrueTransparency"), pDefaultPaintTexture->m_bTrueTransparency);
    if ( m_bTrueTransparency != pDefaultPaintTexture->m_bTrueTransparency )
        bDifferenceToDefault = true;

	//Update 7
	if ( IsOS(OS_VIW7) )
	{
		m_bAlphaTranslucency = iGetx2RCBool(name, (prefix + "TextureAlphaTranslucency"), pDefaultPaintTexture->m_bAlphaTranslucency);
		if ( m_bAlphaTranslucency != pDefaultPaintTexture->m_bAlphaTranslucency )
			bDifferenceToDefault = true;
	}

        //-----------------------------------------------------------------
        //Color
        //-----------------------------------------------------------------

        m_iImageWidth = iGetx2RCInt(name, (prefix + "ImageWidth"), pDefaultPaintTexture->m_iImageWidth);
        m_iImageHeight = iGetx2RCInt(name, (prefix + "ImageHeight"), pDefaultPaintTexture->m_iImageHeight);

        m_strColors = iGetx2RCLine(name, (prefix + "Colors"), pDefaultPaintTexture->m_strColors);
        if ( _stricmp(m_strColors.c_str(), pDefaultPaintTexture->m_strColors.c_str()) != NULL )
            bDifferenceToDefault = true;

        m_strGradientColors = iGetx2RCLine(name, (prefix + "GradientColors"), pDefaultPaintTexture->m_strGradientColors);
        if ( _stricmp(m_strGradientColors.c_str(), pDefaultPaintTexture->m_strGradientColors.c_str()) != NULL )
            bDifferenceToDefault = true;
    
        m_iGradientType = iGetx2RCNamedValue(name, (prefix + "GradientType"), gradientmodeConstants, pDefaultPaintTexture->m_iGradientType);            
        if ( m_iGradientType != pDefaultPaintTexture->m_iGradientType )
            bDifferenceToDefault = true;

        m_iGradientTransformationType = iGetx2RCNamedValue(name, (prefix + "GradientTransformationType"), gradienttransformationConstants, pDefaultPaintTexture->m_iGradientTransformationType);
        if ( m_iGradientTransformationType != pDefaultPaintTexture->m_iGradientTransformationType )
            bDifferenceToDefault = true;
    
        m_strGradientRepeatedColors = iGetx2RCLine(name, (prefix + "GradientRepeated"), pDefaultPaintTexture->m_strGradientRepeatedColors);
        if ( _stricmp(m_strGradientRepeatedColors.c_str(), pDefaultPaintTexture->m_strGradientRepeatedColors.c_str()) != NULL )
            bDifferenceToDefault = true;

        m_strBevels = iGetx2RCLine(name, (prefix + "Bevels"), pDefaultPaintTexture->m_strBevels);
        if ( _stricmp(m_strBevels.c_str(), pDefaultPaintTexture->m_strBevels.c_str()) != NULL )
            bDifferenceToDefault = true;

        if (m_strBevels.empty())
        {
            m_iLeftBevel = iGetx2RCInt(name, (prefix + "LeftBevel"), pDefaultPaintTexture->m_iLeftBevel, 0);
            m_iTopBevel = iGetx2RCInt(name, (prefix + "TopBevel"), pDefaultPaintTexture->m_iTopBevel, 0);
            m_iRightBevel = iGetx2RCInt(name, (prefix + "RightBevel"), pDefaultPaintTexture->m_iRightBevel, 0);
            m_iBottomBevel = iGetx2RCInt(name, (prefix + "BottomBevel"), pDefaultPaintTexture->m_iBottomBevel, 0);
        }
        else
        {
            Parse4SizesCFG(m_strBevels, m_iLeftBevel, m_iRightBevel, m_iTopBevel, m_iBottomBevel, true);
            
            m_iLeftBevel = iGetxRCInt(name, (prefix + "LeftBevel"), m_iLeftBevel, 0);
            m_iTopBevel = iGetxRCInt(name, (prefix + "TopBevel"), m_iTopBevel, 0);
            m_iRightBevel = iGetxRCInt(name, (prefix + "RightBevel"), m_iRightBevel, 0);
            m_iBottomBevel = iGetxRCInt(name, (prefix + "BottomBevel"), m_iBottomBevel, 0);
        }

            if ( m_iLeftBevel != pDefaultPaintTexture->m_iLeftBevel )
                bDifferenceToDefault = true;
            if ( m_iTopBevel != pDefaultPaintTexture->m_iTopBevel )
                bDifferenceToDefault = true;
            if ( m_iRightBevel != pDefaultPaintTexture->m_iRightBevel )
                bDifferenceToDefault = true;
            if ( m_iBottomBevel != pDefaultPaintTexture->m_iBottomBevel )
                bDifferenceToDefault = true;
        
        //-----------------------------------------------------------------
        //Image
        //-----------------------------------------------------------------

        m_strImagePath = iGetx2RCString(name, (prefix + "Image"), pDefaultPaintTexture->m_strImagePath);
        if ( _stricmp(m_strImagePath.c_str(), pDefaultPaintTexture->m_strImagePath.c_str()) != NULL )
            bDifferenceToDefault = true;

        //Fill Image Dimensions, when m_bDisablePerformanceCaching
        if ( m_bDisablePerformanceCaching )
        {
            if ( m_iPaintingMode == 0 )
            {
                HBITMAP tmpmemoryBitmap = LoadLSImageEx(m_strImagePath);

                if (tmpmemoryBitmap)
                {
                    GetLSBitmapSize(tmpmemoryBitmap, &m_iImageWidth, &m_iImageHeight);
                    DeleteObject(tmpmemoryBitmap);
                }
            }
        }

        m_strImageEdges = iGetx2RCLine(name, (prefix + "ImageEdges"), pDefaultPaintTexture->m_strImageEdges);
        if ( _stricmp(m_strImageEdges.c_str(), pDefaultPaintTexture->m_strImageEdges.c_str()) != NULL )
            bDifferenceToDefault = true;

        if (m_strImageEdges.empty())
        {
            m_iLeftImageEdge = iGetx2RCInt(name, (prefix + "ImageLeftEdge"), pDefaultPaintTexture->m_iLeftImageEdge, 0);
            m_iTopImageEdge = iGetx2RCInt(name, (prefix + "ImageTopEdge"), pDefaultPaintTexture->m_iTopImageEdge, 0);
            m_iRightImageEdge = iGetx2RCInt(name, (prefix + "ImageRightEdge"), pDefaultPaintTexture->m_iRightImageEdge, 0);
            m_iBottomImageEdge = iGetx2RCInt(name, (prefix + "ImageBottomEdge"), pDefaultPaintTexture->m_iBottomImageEdge, 0);
        }
        else
        {
            Parse4SizesCFG(m_strImageEdges, m_iLeftImageEdge, m_iRightImageEdge, m_iTopImageEdge, m_iBottomImageEdge, true);
        
            m_iLeftImageEdge = iGetxRCInt(name, (prefix + "ImageLeftEdge"), m_iLeftImageEdge, 0);
            m_iTopImageEdge = iGetxRCInt(name, (prefix + "ImageTopEdge"), m_iTopImageEdge, 0);
            m_iRightImageEdge = iGetxRCInt(name, (prefix + "ImageRightEdge"), m_iRightImageEdge, 0);
            m_iBottomImageEdge = iGetxRCInt(name, (prefix + "ImageBottomEdge"), m_iBottomImageEdge, 0);
        }

            if ( m_iLeftImageEdge != pDefaultPaintTexture->m_iLeftImageEdge )
                bDifferenceToDefault = true;
            if ( m_iTopImageEdge != pDefaultPaintTexture->m_iTopImageEdge )
                bDifferenceToDefault = true;
            if ( m_iRightImageEdge != pDefaultPaintTexture->m_iRightImageEdge )
                bDifferenceToDefault = true;
            if ( m_iBottomImageEdge != pDefaultPaintTexture->m_iBottomImageEdge )
                bDifferenceToDefault = true;

        m_strImageCrop = iGetx2RCLine(name, (prefix + "ImageCrop"), pDefaultPaintTexture->m_strImageCrop);
        if ( _stricmp(m_strImageCrop.c_str(), pDefaultPaintTexture->m_strImageCrop.c_str()) != NULL )
            bDifferenceToDefault = true;

        if (m_strImageCrop.empty())
        {
            m_iLeftImageCrop = iGetx2RCInt(name, (prefix + "ImageLeftCrop"), pDefaultPaintTexture->m_iLeftImageCrop, 0);
            m_iTopImageCrop = iGetx2RCInt(name, (prefix + "ImageTopCrop"), pDefaultPaintTexture->m_iTopImageCrop, 0);
            m_iRightImageCrop = iGetx2RCInt(name, (prefix + "ImageRightCrop"), pDefaultPaintTexture->m_iRightImageCrop, 0);
            m_iBottomImageCrop = iGetx2RCInt(name, (prefix + "ImageBottomCrop"), pDefaultPaintTexture->m_iBottomImageCrop, 0);
        }
        else
        {
            Parse4SizesCFG(m_strImageCrop, m_iLeftImageCrop, m_iRightImageCrop, m_iTopImageCrop, m_iBottomImageCrop, true);
        
            m_iLeftImageCrop = iGetxRCInt(name, (prefix + "ImageLeftCrop"), m_iLeftImageCrop, 0);
            m_iTopImageCrop = iGetxRCInt(name, (prefix + "ImageTopCrop"), m_iTopImageCrop, 0);
            m_iRightImageCrop = iGetxRCInt(name, (prefix + "ImageRightCrop"), m_iRightImageCrop, 0);
            m_iBottomImageCrop = iGetxRCInt(name, (prefix + "ImageBottomCrop"), m_iBottomImageCrop, 0);
        }

            if ( m_iLeftImageCrop != pDefaultPaintTexture->m_iLeftImageCrop )
                bDifferenceToDefault = true;
            if ( m_iTopImageCrop != pDefaultPaintTexture->m_iTopImageCrop )
                bDifferenceToDefault = true;
            if ( m_iRightImageCrop != pDefaultPaintTexture->m_iRightImageCrop )
                bDifferenceToDefault = true;
            if ( m_iBottomImageCrop != pDefaultPaintTexture->m_iBottomImageCrop )
                bDifferenceToDefault = true;

        m_iMode = iGetx2RCNamedValue(name, (prefix + "ImageMode"), modeConstants, pDefaultPaintTexture->m_iMode);
        if ( m_iMode != pDefaultPaintTexture->m_iMode )
            bDifferenceToDefault = true;

        //-----------------------------------------------------------------
        //Icon
        //-----------------------------------------------------------------

        m_strIconPath = iGetx2RCString(name, (prefix + "Icon"), pDefaultPaintTexture->m_strIconPath);
        if ( _stricmp(m_strIconPath.c_str(), pDefaultPaintTexture->m_strIconPath.c_str()) != NULL )
            bDifferenceToDefault = true;

        m_iExtractIconSize = iGetx2RCInt(name, (prefix + "IconExtractionSize"), pDefaultPaintTexture->m_iExtractIconSize, 16, 256);
        if ( m_iExtractIconSize != pDefaultPaintTexture->m_iExtractIconSize )
            bDifferenceToDefault = true;
    
        m_iIconMode = iGetx2RCNamedValue(name, (prefix + "IconMode"), modeConstants, pDefaultPaintTexture->m_iIconMode);
        if ( m_iIconMode != pDefaultPaintTexture->m_iIconMode )
            bDifferenceToDefault = true;

        if ( m_iPaintingMode == 1 )
        {   
            //Fill Image Dimensions, when m_bDisablePerformanceCaching
            if ( m_bDisablePerformanceCaching )
            {
                m_iImageWidth = m_iExtractIconSize;
                m_iImageHeight = m_iExtractIconSize;
            }

            //Disable ImageEdges
            m_iLeftImageEdge = 0;
            m_iRightImageEdge = 0;
            m_iTopImageEdge = 0;
            m_iBottomImageEdge = 0;
        }

        //-----------------------------------------------------------------
        //Image/Icon
        //-----------------------------------------------------------------

        m_clrHueColor = iGetx2RCColor(name, (prefix + "HueColor"), pDefaultPaintTexture->m_clrHueColor);
        if ( m_clrHueColor != pDefaultPaintTexture->m_clrHueColor )
            bDifferenceToDefault = true;
        
        m_iHueIntensity = iGetx2RCInt(name, (prefix + "HueIntensity"), pDefaultPaintTexture->m_iHueIntensity, 0, 100);
        if ( m_iHueIntensity != pDefaultPaintTexture->m_iHueIntensity )
            bDifferenceToDefault = true;

        m_iLuminanceIntensity = iGetx2RCInt(name, (prefix + "LuminanceIntensity"), pDefaultPaintTexture->m_iLuminanceIntensity, -100, 100);
        if ( m_iLuminanceIntensity != pDefaultPaintTexture->m_iLuminanceIntensity )
            bDifferenceToDefault = true;

        m_iSaturationIntensity = iGetx2RCInt(name, (prefix + "SaturationIntensity"), pDefaultPaintTexture->m_iSaturationIntensity, 0, 200);
        if ( m_iSaturationIntensity != pDefaultPaintTexture->m_iSaturationIntensity )
            bDifferenceToDefault = true;

        m_clrMixColor = iGetx2RCColor(name, (prefix + "MixColor"), pDefaultPaintTexture->m_clrMixColor);                
        if ( m_clrMixColor != pDefaultPaintTexture->m_clrMixColor )
            bDifferenceToDefault = true;
        
        m_iMixIntensity = iGetx2RCInt(name, (prefix + "MixIntensity"), pDefaultPaintTexture->m_iMixIntensity, 0, 100);
        if ( m_iMixIntensity != pDefaultPaintTexture->m_iMixIntensity )
            bDifferenceToDefault = true;

    //Update 2
    //-----------------------------------------------------------------
    m_iShapeMode = iGetx2RCNamedValue(name, (prefix + "ShapeType"), shapemodeConstants, pDefaultPaintTexture->m_iShapeMode);
    if ( m_iShapeMode != pDefaultPaintTexture->m_iShapeMode )
        bDifferenceToDefault = true;

    m_strRoundedEdges = iGetx2RCLine(name, (prefix + "RoundedEdges"), pDefaultPaintTexture->m_strRoundedEdges);
    if ( _stricmp(m_strRoundedEdges.c_str(), pDefaultPaintTexture->m_strRoundedEdges.c_str()) != NULL )
        bDifferenceToDefault = true;
        
    Parse4SizesCFG(m_strRoundedEdges, m_iRounded_X, m_iRounded_X, m_iRounded_Y, m_iRounded_Y, true);

    m_iRounded_X = m_iRounded_X*2;
    m_iRounded_Y = m_iRounded_Y*2;

    //Update 4
    //-----------------------------------------------------------------
    m_strTexture_Rotation = iGetx2RCLine(name, (prefix + "TextureRotation"), pDefaultPaintTexture->m_strTexture_Rotation);
    if ( _stricmp(m_strTexture_Rotation.c_str(), pDefaultPaintTexture->m_strTexture_Rotation.c_str()) != NULL )
        bDifferenceToDefault = true;

    ParseRotationCFG(m_strTexture_Rotation, m_iTexture_Rotation, m_iRotationWidth, m_iRotationHeight);

    if ( m_iTexture_Rotation != 0 )
    {
        if ( m_iPaintingMode == 2 || m_iPaintingMode == 3 || m_iPaintingMode == 4 || m_iPaintingMode == 6 )
        {
            if ( m_iRotationWidth > 0 && m_iRotationHeight > 0 )
            {
                m_iImageWidth = m_iRotationWidth;
                m_iImageHeight = m_iRotationHeight;
            }
        }
        else if ( m_iPaintingMode == 5 ) 
        {
            MessageBox(NULL, (name + prefix + "TextureRotation doesn't work with the PaintingMode:\n\".animation\"").c_str(), "Texture Rotation Failure", MB_OK | MB_ICONWARNING | MB_TOPMOST | MB_SETFOREGROUND);
            m_iTexture_Rotation = 0;
        }
    }

    //Update 5
    //-----------------------------------------------------------------
    m_strInternalMSStyle = iGetx2RCLine(name, (prefix + "StyleDefinition"), pDefaultPaintTexture->m_strInternalMSStyle);
    
    ParseStyleCFG(m_strInternalMSStyle, m_strMSStyle_Class, m_iMSStyle_Parts, m_iMSStyle_States);

    if ( _stricmp(m_strInternalMSStyle.c_str(), pDefaultPaintTexture->m_strInternalMSStyle.c_str()) != NULL )
        bDifferenceToDefault = true;

    if ( m_iPaintingMode == 6 )
    {
        if ( m_strInternalMSStyle.empty() || m_strMSStyle_Class.empty() )
        {
            MessageBox(NULL, (name + prefix + "StyleDefinition is missing!\nPaintingMode: \".visualstyle\" requires a correct \"StyleDefinition\"!").c_str(), "StyleDefinition Failure", MB_OK | MB_ICONWARNING | MB_TOPMOST | MB_SETFOREGROUND);
            m_iPaintingMode = 2;
        }
    }

    //-----------------------------------------------------------------
    //Required Preparation Colors
    //-----------------------------------------------------------------
    if ( m_iPaintingMode == 2 || m_iPaintingMode == 3 || m_iPaintingMode == 6 )
    {
        if ( m_iPaintingMode == 6 )
            m_bUseAlphaMap = true;

        ParseBGBevelColorCFG(m_strColors, m_clrBackgroundColor, m_clrLightBevelColor, m_clrDarkBevelColor);

        if ( m_iPaintingMode == 2 || m_iPaintingMode == 6 )
        {
            //BG Insert in Gradient List
            GradientColors.insert(GradientColors.end(), m_clrBackgroundColor);
            
            if ( !m_strGradientColors.empty() )
            {   
                CStr strProcessing = m_strGradientColors.c_str();
                strProcessing.Replace("\"", "");

                int color = 0;
                COLORREF insertColor = m_clrBackgroundColor;
                char token[4096] = {0};
                LPCSTR nextToken = strProcessing;

                while (GetToken(nextToken, token, &nextToken, false))
                {
                    ProcessColor(token, insertColor);
            
                    GradientColors.insert(GradientColors.end(), insertColor );
                }
            }

            if ( !m_strGradientRepeatedColors.empty() )
            {   
                CStr strProcessing = m_strGradientRepeatedColors.c_str();
                strProcessing.Replace("\"", "");

                char token[4096] = {0};
                LPCSTR nextToken = strProcessing;

                while (GetToken(nextToken, token, &nextToken, false))
                    GradientRepeatedColors.insert(GradientRepeatedColors.end(), atoi(token) );
            }
        }
    }
    //-----------------------------------------------------------------
    //Required Preparation Image
    //-----------------------------------------------------------------
    else if ( m_iPaintingMode == 0 && !m_bDisablePerformanceCaching )
    {
        Create_Image();
    }
    //-----------------------------------------------------------------
    //Required Preparation Icon
    //-----------------------------------------------------------------
    else if ( m_iPaintingMode == 1 && !m_bDisablePerformanceCaching )
    {
        Create_Icon();
    }
    else if ( m_iPaintingMode == 5 )
    {
        string strFramesFile = iGetx2RCString(name, (prefix + "Frames"), "");

        //Check for Single Image animations
        if ( !strFramesFile.empty() )
        {
            HBITMAP tmpmemoryBitmap = LoadLSImageEx(strFramesFile);

            if (tmpmemoryBitmap)
            {   
                GetLSBitmapSize(tmpmemoryBitmap, &m_iImageWidth, &m_iImageHeight);

                BITMAP bm;
                GetObject(tmpmemoryBitmap, sizeof(BITMAP), &bm);

                bool bImageHasAlphaChannel = (bm.bmBitsPixel >= 32 ? true : false);
                        
                CStr strFramesPath = strFramesFile.c_str();
                strFramesPath.Lower();

                if ( GetFileType(strFramesFile) != 1 )
                    bImageHasAlphaChannel = false;
        
                HDC tmpmemoryDC = CreateCompatibleDC(NULL);
                HBITMAP oldtmpmemoryBMP = (HBITMAP) SelectObject(tmpmemoryDC, tmpmemoryBitmap);

                memoryDC = CreateCompatibleDC(NULL);

                BITMAPINFO alpha;
                ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
                alpha.bmiHeader.biWidth=m_iImageWidth;      // Set size you need
                alpha.bmiHeader.biHeight=m_iImageHeight;    // Set size you need
                alpha.bmiHeader.biPlanes=1;
                alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
                alpha.bmiHeader.biSizeImage=0;
                alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
                alpha.bmiHeader.biClrUsed= 0;
                alpha.bmiHeader.biClrImportant= 0;
                VOID *pvBits;

                memoryBitmap = CreateDIBSection( memoryDC,
                                                    &alpha,
                                                    DIB_RGB_COLORS,
                                                    &pvBits,
                                                    NULL,
                                                    0 );

                SelectObject(memoryDC, memoryBitmap);

                BitBlt(memoryDC, 0, 0, m_iImageWidth, m_iImageHeight, tmpmemoryDC, 0, 0, SRCCOPY);

                tmpmemoryBitmap = (HBITMAP) SelectObject(tmpmemoryDC, oldtmpmemoryBMP);
                DeleteDC(tmpmemoryDC);
                DeleteObject(tmpmemoryBitmap);
                DeleteObject(oldtmpmemoryBMP);

                m_iFrameWidth = iGetx2RCInt(name, (prefix + "FrameWidth"), 0);
                m_iFrameHeight = iGetx2RCInt(name, (prefix + "FrameHeight"), 0);
                
                PrepareImageEffects(memoryBitmap, bImageHasAlphaChannel, m_clrHueColor, m_iHueIntensity, m_iLuminanceIntensity, m_iSaturationIntensity, m_clrMixColor, m_iMixIntensity);
            
                if ( !m_bUseAlphaMap && bImageHasAlphaChannel )
                    m_bUseAlphaMap = true;
            }
        }
        //Check for Multi Image animations
        else
        {
            StringList listAnimation;
            
            LPVOID f = LCOpen(NULL);

            if(f != NULL)
            {
                char lineBuffer[MAX_LINE_LENGTH] = {0};

                while(LCReadNextConfig(f, ("*" + name + prefix + "Frames").c_str(), lineBuffer, MAX_LINE_LENGTH))
                {
                    char temp[64], configline[MAX_LINE_LENGTH];
                    char *buffers[] = {temp, configline};
                        
                    if (LCTokenize(lineBuffer, buffers, 2, NULL) >= 2)
                        listAnimation.insert(listAnimation.end(), (string) configline);
                }
                    
                LCClose(f);
            }

            if ( !listAnimation.empty() )
            {
                m_iFrameWidth = iGetx2RCInt(name, (prefix + "FrameWidth"), 0);
                m_iFrameHeight = iGetx2RCInt(name, (prefix + "FrameHeight"), 0);
        
                m_iImageWidth = m_iFrameWidth*listAnimation.size();
                m_iImageHeight = m_iFrameHeight*listAnimation.size();

                memoryDC = CreateCompatibleDC(NULL);

                BITMAPINFO alpha;
                ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
                alpha.bmiHeader.biWidth=m_iImageWidth;      // Set size you need
                alpha.bmiHeader.biHeight=m_iImageHeight;    // Set size you need
                alpha.bmiHeader.biPlanes=1;
                alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
                alpha.bmiHeader.biSizeImage=0;
                alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
                alpha.bmiHeader.biClrUsed= 0;
                alpha.bmiHeader.biClrImportant= 0;
                VOID *pvBits;

                memoryBitmap = CreateDIBSection( memoryDC,
                                                    &alpha,
                                                    DIB_RGB_COLORS,
                                                    &pvBits,
                                                    NULL,
                                                    0 );

                SelectObject(memoryDC, memoryBitmap);

                HBITMAP tmpmemoryBitmap = NULL;
                HDC tmpmemoryDC = NULL;
                HBITMAP oldtmpmemoryBMP = NULL;

                bool bImageHasAlphaChannel = true;

                int x = 0;
                    
                for (StringListIterator sit = listAnimation.begin(); sit != listAnimation.end(); sit++)
                {
                    tmpmemoryBitmap = LoadLSImageEx((*sit));

                    if (tmpmemoryBitmap)
                    {
                        tmpmemoryDC = CreateCompatibleDC(NULL);
                        oldtmpmemoryBMP = (HBITMAP) SelectObject(tmpmemoryDC, tmpmemoryBitmap);

                        BITMAP bm;
                        GetObject(tmpmemoryBitmap, sizeof(BITMAP), &bm);

                        if ( bImageHasAlphaChannel )
                        {   
                            bImageHasAlphaChannel = (bm.bmBitsPixel >= 32 ? true : false);
                                
                            if ( GetFileType((*sit)) != 1 )
                                bImageHasAlphaChannel = false;
                        }

                        BitBlt(memoryDC, x, 0, m_iFrameWidth, m_iFrameHeight, tmpmemoryDC, 0, 0, SRCCOPY);

                        x = x+m_iFrameWidth;

                        tmpmemoryBitmap = (HBITMAP) SelectObject(tmpmemoryDC, oldtmpmemoryBMP);
                        DeleteDC(tmpmemoryDC);
                        DeleteObject(tmpmemoryBitmap);
                        DeleteObject(oldtmpmemoryBMP);
                    }
                }

                PrepareImageEffects(memoryBitmap, bImageHasAlphaChannel, m_clrHueColor, m_iHueIntensity, m_iLuminanceIntensity, m_iSaturationIntensity, m_clrMixColor, m_iMixIntensity);

                if ( !m_bUseAlphaMap && bImageHasAlphaChannel )
                    m_bUseAlphaMap = true;
            }
        }
    }

    if ( bDefaultPaintTextureCreated )
    {
        delete pDefaultPaintTexture;
        pDefaultPaintTexture = NULL;
    }

    if ( m_iPaintingMode == -1 )
        return false;
    else if ( bDifferenceToDefault || bDefaultPaintTextureCreated )
        return 1;
    else
        return 2;
}

void PaintTexture::apply(HDC desthDC, int x, int y, int width, int height, int iFrameCounter)
{
    if ( width <= 0 || height <= 0 || m_iPaintingMode == -1 || (!m_bDisablePerformanceCaching && memoryBitmap == 0 && (m_iPaintingMode == 0 || m_iPaintingMode == 1 || m_iPaintingMode == 5)) )
    {
		return;
	}

    int iSpecialPosX = 0;
    int iSpecialPosY = 0;

    int iSpecialWidth = width;
    int iSpecialHeight = height;

    if ( m_bSpecialSetup )
    {
        iSpecialPosX = xParseCoord(m_strTexture_X.c_str(), 0, width, false);
        iSpecialPosY = xParseCoord(m_strTexture_Y.c_str(), 0, height, true);

        iSpecialWidth = xParseDimen(m_strTexture_Width.c_str(), 0, width, false);
        iSpecialHeight = xParseDimen(m_strTexture_Height.c_str(), 0, height, true);
    }

    int iPaintedWidth = iSpecialWidth;
    int iPaintedHeight = iSpecialHeight;

    if ( m_iTexture_Rotation != 0 && m_iRotationWidth > 0 && m_iRotationHeight > 0 )
    {
        iPaintedWidth = m_iRotationWidth;
        iPaintedHeight = m_iRotationHeight;
    }

    if ( (m_iPaintingMode == 2 || m_iPaintingMode == 3 || m_iPaintingMode == 6) && (iPaintedWidth != m_iOldPaintingWidth || iPaintedHeight != m_iOldPaintingHeight) )
    {
        if(memoryBitmap)
        {
            memoryBitmap = (HBITMAP) SelectObject(memoryDC, memoryBitmap);
            DeleteObject(memoryBitmap);
        }
        if(memoryDC)
            DeleteDC(memoryDC);

        memoryBitmap = NULL;        
        memoryDC = NULL;

        m_iOldPaintingWidth = iPaintedWidth;
        m_iOldPaintingHeight = iPaintedHeight;
    }

    HDC bufferDC = CreateCompatibleDC(NULL);

    BITMAPINFO alpha;
    ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
    alpha.bmiHeader.biWidth=iPaintedWidth;      // Set size you need
    alpha.bmiHeader.biHeight=iPaintedHeight;    // Set size you need
    alpha.bmiHeader.biPlanes=1;
    alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
    alpha.bmiHeader.biSizeImage=0;
    alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
    alpha.bmiHeader.biClrUsed= 0;
    alpha.bmiHeader.biClrImportant= 0;
    VOID *pvBits;

    HBITMAP bufferBitmap = CreateDIBSection( bufferDC,
                                                &alpha,
                                                DIB_RGB_COLORS,
                                                &pvBits,
                                                NULL,
                                                0 );

    HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

    SetStretchBltMode(bufferDC, STRETCH_DELETESCANS);

    if (m_iPaintingMode == 2 || m_iPaintingMode == 3 || m_iPaintingMode == 6)
    {
        //Use Backup, if existant
        if (memoryBitmap != NULL && memoryDC != NULL)
        {
            BitBlt(bufferDC, 0, 0, iPaintedWidth, iPaintedHeight, memoryDC, 0, 0, SRCCOPY);
        }
        else
        {   
            RECT r;
            r.left = 0;
            r.top = 0;
            r.right = iPaintedWidth;
            r.bottom = iPaintedHeight;

            bool bMSStyleFailed = false;

            if (m_iPaintingMode == 6)
            {
                if ( InitThemes() )
                {
                    BITMAP bm;
                    GetObject(bufferBitmap, sizeof(BITMAP), &bm);

                    BYTE *pPixel = (LPBYTE)bm.bmBits;

                    int iSize = iPaintedWidth * iPaintedHeight;

                    for (int j = 0; j < iSize; j++)
                    {
                        pPixel[0] = 0;
                        pPixel[1] = 0;
                        pPixel[2] = 0;
                        pPixel[3] = 1;
                        
                        pPixel+= 4;
                    }

                    HTHEME hTheme = zOpenThemeData(NULL, (LPCWSTR) m_strMSStyle_Class.c_str());

                    if (hTheme)
                    {
                        HRESULT hr;
                        SIZE sRealSize;
                        zGetThemePartSize(hTheme, bufferDC, m_iMSStyle_Parts, m_iMSStyle_States, &r, TS_TRUE, &sRealSize);

                        if ( CompareRect(r, sRealSize) )
                            hr = zDrawThemeBackground(hTheme, bufferDC, m_iMSStyle_Parts, m_iMSStyle_States, &r, NULL);
                        else
                            hr = Create_hThemeImage(hTheme, bufferDC, m_iMSStyle_Parts, m_iMSStyle_States, sRealSize, r);
        
                        if ( hr != S_OK )
                        {                       
                            zGetThemePartSize(hTheme, bufferDC, m_iMSStyle_Parts, 0, &r, TS_TRUE, &sRealSize);

                            if ( CompareRect(r, sRealSize) )
                                hr = zDrawThemeBackground(hTheme, bufferDC, m_iMSStyle_Parts, 0, &r, NULL);
                            else
                                hr = Create_hThemeImage(hTheme, bufferDC, m_iMSStyle_Parts, 0, sRealSize, r);
            
                            if ( hr != S_OK )
                            {
                                zGetThemePartSize(hTheme, bufferDC, m_iMSStyle_Parts, 1, &r, TS_TRUE, &sRealSize);

                                if ( CompareRect(r, sRealSize) )
                                    hr = zDrawThemeBackground(hTheme, bufferDC, m_iMSStyle_Parts, 1, &r, NULL);
                                else
                                    hr = Create_hThemeImage(hTheme, bufferDC, m_iMSStyle_Parts, 1, sRealSize, r);
                
                                if ( hr != S_OK )
                                {
                                    zGetThemePartSize(hTheme, bufferDC, 0, 0, &r, TS_TRUE, &sRealSize);

                                    if ( CompareRect(r, sRealSize) )
                                        hr = zDrawThemeBackground(hTheme, bufferDC, 0, 0, &r, NULL);
                                    else
                                        hr = Create_hThemeImage(hTheme, bufferDC, 0, 0, sRealSize, r);
                    
                                    if ( hr != S_OK )
                                    {
                                        bMSStyleFailed = true;

                                        m_bUseAlphaMap = false;
                                    }
                                }
                            }
                        }
                    
                        zCloseThemeData(hTheme);
                        hTheme = NULL;
                    }
                    else
                    {
                        MessageBox(NULL, ("HTHEME creation failed!\nPossible CLASS problem, check your setup!\n\nStyleDefinition:\n" + m_strInternalMSStyle).c_str(), "HTHEME creation failed!", MB_OK | MB_ICONWARNING | MB_TOPMOST | MB_SETFOREGROUND);

                        bMSStyleFailed = true;

                        m_bUseAlphaMap = false;
                    }
                }
                else
                {
                    MessageBox(NULL, "Dll initialization failed! Check your uxtheme.dll!", "uxtheme.dll Error!", MB_OK | MB_ICONWARNING | MB_TOPMOST | MB_SETFOREGROUND);

                    bMSStyleFailed = true;

                    m_bUseAlphaMap = false;
                }
            }

            if (m_iPaintingMode == 2 || m_iPaintingMode == 3 || bMSStyleFailed)
            {
                if ( m_iShapeMode != 1 )
                {
                    ::SetBkColor(bufferDC, RGB(255,0,255));
                    ::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

                    HRGN rgnForm = NULL;

                    if ( m_iShapeMode == 3 )
                        rgnForm = CreateRoundRectRgn(r.left, r.top, r.right+1, r.bottom+1, m_iRounded_X, m_iRounded_Y);
                    else
                        rgnForm = CreateEllipticRgn(r.left, r.top, r.right+1, r.bottom+1);

                    SelectClipRgn(bufferDC, rgnForm);

                    DeleteObject(rgnForm);
                    rgnForm = NULL;
                }

                //Colors
                if (m_iPaintingMode == 2 || bMSStyleFailed)
                {
                    int bevelWidth = m_iLeftBevel+m_iRightBevel;
                    int bevelHeight = m_iTopBevel+m_iBottomBevel;

                    // fill in the background
                    if (bevelWidth+bevelHeight > 0 && (bevelWidth < iPaintedWidth && bevelHeight < iPaintedHeight))
                    {
                        ::SetBkColor(bufferDC, m_clrDarkBevelColor);
                        ::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

                        r.right = r.right-m_iRightBevel;
                        r.bottom = r.bottom-m_iBottomBevel;

                        if ( m_iShapeMode != 1 )
                        {
                            HRGN rgnForm = NULL;

                            if ( m_iShapeMode == 3 )
                                rgnForm = CreateRoundRectRgn(r.left, r.top, r.right+1, r.bottom+1, m_iRounded_X, m_iRounded_Y);
                            else
                                rgnForm = CreateEllipticRgn(r.left, r.top, r.right+1, r.bottom+1);
                        
                            SelectClipRgn(bufferDC, rgnForm);

                            DeleteObject(rgnForm);
                            rgnForm = NULL;
                        }

                        ::SetBkColor(bufferDC, m_clrLightBevelColor);
                        ::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

                        r.left = r.left+m_iLeftBevel;
                        r.top = r.top+m_iTopBevel;

                        if ( m_iShapeMode != 1 )
                        {
                            HRGN rgnForm = NULL;

                            if ( m_iShapeMode == 3 )
                                rgnForm = CreateRoundRectRgn(r.left, r.top, r.right+1, r.bottom+1, m_iRounded_X, m_iRounded_Y);
                            else
                                rgnForm = CreateEllipticRgn(r.left, r.top, r.right+1, r.bottom+1);
                    
                            SelectClipRgn(bufferDC, rgnForm);

                            DeleteObject(rgnForm);
                            rgnForm = NULL;
                        }

                        if ( GradientColors.size() > 1 && m_iGradientType != GRADIENT_NONE )
                        {
                            if ( m_iGradientType == GRADIENT_HORIZONTAL && m_iGradientTransformationType == TRANSFORMATION_NONE )
                                DrawMultiGradient(bufferDC, r, GradientColors, GradientRepeatedColors, false);
                            else if ( m_iGradientType == GRADIENT_VERTICAL && m_iGradientTransformationType == TRANSFORMATION_NONE )
                                DrawMultiGradient(bufferDC, r, GradientColors, GradientRepeatedColors, true);
                            else
                                DrawSpecialGradient(bufferDC, r, GradientColors[0], GradientColors[1], m_iGradientType, m_iGradientTransformationType);
                        }
                        else
                        {
                            ::SetBkColor(bufferDC, m_clrBackgroundColor);
                            ::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
                        }
                    }
                    else
                    {
                        if ( GradientColors.size() > 1 && m_iGradientType != GRADIENT_NONE )
                        {
                            if ( m_iGradientType == GRADIENT_HORIZONTAL && m_iGradientTransformationType == TRANSFORMATION_NONE )
                                DrawMultiGradient(bufferDC, r, GradientColors, GradientRepeatedColors, false);
                            else if ( m_iGradientType == GRADIENT_VERTICAL && m_iGradientTransformationType == TRANSFORMATION_NONE )
                                DrawMultiGradient(bufferDC, r, GradientColors, GradientRepeatedColors, true);
                            else
                                DrawSpecialGradient(bufferDC, r, GradientColors[0], GradientColors[1], m_iGradientType, m_iGradientTransformationType);
                        }
                        else
                        {
                            ::SetBkColor(bufferDC, m_clrBackgroundColor);
                            ::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
                        }
                    }
                    
                    if ( m_iShapeMode != 1 )
                        SelectClipRgn(bufferDC, NULL);
                }
                //PlainColor
                else
                {
                    ::SetBkColor(bufferDC, m_clrBackgroundColor);
                    ::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
                }
            }

            //Create Backup, if not existant
            if (!m_bDisablePerformanceCaching && memoryBitmap == NULL && memoryDC == NULL)
            {
                memoryDC = CreateCompatibleDC(NULL);

                BITMAPINFO alpha;
                ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
                alpha.bmiHeader.biWidth=iPaintedWidth;      // Set size you need
                alpha.bmiHeader.biHeight=iPaintedHeight;    // Set size you need
                alpha.bmiHeader.biPlanes=1;
                alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
                alpha.bmiHeader.biSizeImage=0;
                alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
                alpha.bmiHeader.biClrUsed= 0;
                alpha.bmiHeader.biClrImportant= 0;
                VOID *pvBits;

                memoryBitmap = CreateDIBSection( memoryDC,
                                                    &alpha,
                                                    DIB_RGB_COLORS,
                                                    &pvBits,
                                                    NULL,
                                                    0 );

                SelectObject(memoryDC, memoryBitmap);
            
                SetStretchBltMode(memoryDC, STRETCH_DELETESCANS);

                BitBlt(memoryDC, 0, 0, iPaintedWidth, iPaintedHeight, bufferDC, 0, 0, SRCCOPY);
            }
        }
    }
    //Image
    else if (m_iPaintingMode == 0)
    {
        if ( !m_bDisablePerformanceCaching )
        {
            MultiBlt(bufferDC, 0, 0, iPaintedWidth, iPaintedHeight,
                            memoryDC, 0, 0, m_iImageWidth, m_iImageHeight,
                            m_iLeftImageEdge, m_iTopImageEdge, m_iRightImageEdge, m_iBottomImageEdge,
                            m_iMode, SRCCOPY);
        }
        else
        {
            if ( Create_Image() )
            {
                MultiBlt(bufferDC, 0, 0, iPaintedWidth, iPaintedHeight,
                            memoryDC, 0, 0, m_iImageWidth, m_iImageHeight,
                            m_iLeftImageEdge, m_iTopImageEdge, m_iRightImageEdge, m_iBottomImageEdge,
                            m_iMode, SRCCOPY);

                if(memoryBitmap)
                {
                    memoryBitmap = (HBITMAP) SelectObject(memoryDC, memoryBitmap);
                    DeleteObject(memoryBitmap);
                }
                if(memoryDC)
                    DeleteDC(memoryDC);

                memoryBitmap = NULL;
                memoryDC = NULL;
            }
        }
    }
    //Icon
    else if (m_iPaintingMode == 1)
    {
        if ( !m_bDisablePerformanceCaching )
        {
            MultiBlt(bufferDC, 0, 0, iPaintedWidth, iPaintedHeight,
                        memoryDC, 0, 0, m_iImageWidth, m_iImageHeight,
                        0, 0, 0, 0,
                        m_iIconMode, SRCCOPY);
        }
        else
        {   
            if ( Create_Icon() )
            {
                MultiBlt(bufferDC, 0, 0, iPaintedWidth, iPaintedHeight,
                            memoryDC, 0, 0, m_iImageWidth, m_iImageHeight,
                            0, 0, 0, 0,
                            m_iIconMode, SRCCOPY);

                if(memoryBitmap)
                {
                    memoryBitmap = (HBITMAP) SelectObject(memoryDC, memoryBitmap);
                    DeleteObject(memoryBitmap);
                }
                if(memoryDC)
                    DeleteDC(memoryDC);

                memoryBitmap = NULL;
                memoryDC = NULL;
            }
        }
    }
    else if (m_iPaintingMode == 5 && iFrameCounter != -1)
    {
        MultiBlt(bufferDC, 0, 0, iPaintedWidth, iPaintedHeight,
                        memoryDC, (iFrameCounter-1)*m_iFrameWidth, 0, m_iFrameWidth, m_iFrameHeight,
                        m_iLeftImageEdge, m_iTopImageEdge, m_iRightImageEdge, m_iBottomImageEdge,
                        m_iMode, SRCCOPY);
    }   

    if ( m_iBorderMethod != 0 || m_iPaintingMode == 4 )
    {
        RECT completeFillRect = {0, 0, iPaintedWidth, iPaintedHeight};

        if (m_iBorderMethod == 1)
            DrawEdge(bufferDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT | (m_iPaintingMode == 4 ? BF_MIDDLE : 0));
        else if (m_iBorderMethod == 2)
            DrawEdge(bufferDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT | (m_iPaintingMode == 4 ? BF_MIDDLE : 0));
        else if (m_iBorderMethod == 3)
            DrawEdge(bufferDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT | (m_iPaintingMode == 4 ? BF_MIDDLE : 0));
        else if (m_iBorderMethod == 4)
            DrawEdge(bufferDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT | (m_iPaintingMode == 4 ? BF_MIDDLE : 0));
        else if (m_iPaintingMode == 4)
            DrawEdge(bufferDC, &completeFillRect, EDGE_RAISED, BF_SOFT | BF_MIDDLE);
    }

    BITMAP bm;
    GetObject(bufferBitmap, sizeof(BITMAP), &bm);

    BYTE *pPixel = (LPBYTE)bm.bmBits;

    int iSize = bm.bmHeight * bm.bmWidth;

    for (int j = 0; j < iSize; j++)
    {
        //Remove Magic Pink
        if ( pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255 )
        {
            if ( m_bTrueTransparency || m_bAlphaTranslucency )
            {   
                pPixel[0] = 255;
                pPixel[1] = 0;
                pPixel[2] = 255;
                pPixel[3] = 255;
            }
            else
            {
                pPixel[0] = 0;
                pPixel[1] = 0;
                pPixel[2] = 0;
                pPixel[3] = 0;
            }
        }
        else if ( !m_bUseAlphaMap || m_iPaintingMode == 2 || m_iPaintingMode == 3 || m_iPaintingMode == 4 ) 
        {
            if ( m_iPaintingMode != 1 )
                pPixel[3] = 255;
        }
        else if ( m_bUseAlphaMap && pPixel[3] == 0 )
        {
            if ( (m_iPaintingMode != 1 && m_iMode <= 3) || (m_iPaintingMode == 1 && m_iIconMode <= 3) )
            {
                pPixel[3] = 255;
            }
            else
            {
                if ( m_iBorderMethod == 0 )
                {
                    pPixel[0] = 0;
                    pPixel[1] = 0;
                    pPixel[2] = 0;
                    pPixel[3] = 0;
                }
                //ToDo Simple hack, but work 90% of the time
                else  
                {
                    if ( pPixel[0] >= 5 && pPixel[1] >= 5 && pPixel[2] >= 5 )
                    {
                        pPixel[3] = 255;
                    }
                    else
                    {
                        pPixel[0] = 0;
                        pPixel[1] = 0;
                        pPixel[2] = 0;
                        pPixel[3] = 0;
                    }
                }
            }
        }
        else if ( m_bUseAlphaMap && pPixel[3] == 1 )
        {
            pPixel[0] = 0;
            pPixel[1] = 0;
            pPixel[2] = 0;
            pPixel[3] = 0;
        }

        pPixel+= 4;
    }

    if ( m_iTexture_Rotation != 0 )
    {
        //Rotation
        HBITMAP tmpbufferBitmap = aarot::rotate(bufferBitmap, m_iTexture_Rotation);

        GetLSBitmapSize(tmpbufferBitmap, &iPaintedWidth, &iPaintedHeight);

        bufferBitmap = (HBITMAP) SelectObject(bufferDC, tmpbufferBitmap);
        DeleteObject(bufferBitmap);
        bufferBitmap = NULL;

        bufferBitmap = (HBITMAP) CopyImage(tmpbufferBitmap, IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION);

        tmpbufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);
        DeleteObject(tmpbufferBitmap);
        tmpbufferBitmap = NULL;
    }

    BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, m_iAlphaTransparency, (m_bTrueTransparency ? 0 : AC_SRC_ALPHA) };

    AlphaBlend(desthDC, x+iSpecialPosX, y+iSpecialPosY, iSpecialWidth, iSpecialHeight, bufferDC, 0, 0, iPaintedWidth, iPaintedHeight, blendFunction);

    bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
    DeleteDC(bufferDC); 
    DeleteObject(bufferBitmap);
    DeleteObject(oldBMP);
}

bool PaintTexture::Create_Image()
{
    if(memoryBitmap)
    {
        memoryBitmap = (HBITMAP) SelectObject(memoryDC, memoryBitmap);
        DeleteObject(memoryBitmap);
    }
    if(memoryDC)
        DeleteDC(memoryDC);

    memoryBitmap = NULL;
    memoryDC = NULL;
    
    HBITMAP tmpmemoryBitmap = LoadLSImageEx(m_strImagePath);

    if (tmpmemoryBitmap)
    {   
        GetLSBitmapSize(tmpmemoryBitmap, &m_iImageWidth, &m_iImageHeight);

        BITMAP bm;
        GetObject(tmpmemoryBitmap, sizeof(BITMAP), &bm);

        bool bImageHasAlphaChannel = (bm.bmBitsPixel >= 32 ? true : false);

        if ( GetFileType(m_strImagePath) != 1 )
            bImageHasAlphaChannel = false;
    
        HDC tmpmemoryDC = CreateCompatibleDC(NULL);
        HBITMAP oldtmpmemoryBMP = (HBITMAP) SelectObject(tmpmemoryDC, tmpmemoryBitmap);

        memoryDC = CreateCompatibleDC(NULL);

        BITMAPINFO alpha;
        ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
        alpha.bmiHeader.biWidth=m_iImageWidth;      // Set size you need
        alpha.bmiHeader.biHeight=m_iImageHeight;    // Set size you need
        alpha.bmiHeader.biPlanes=1;
        alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
        alpha.bmiHeader.biSizeImage=0;
        alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
        alpha.bmiHeader.biClrUsed= 0;
        alpha.bmiHeader.biClrImportant= 0;
        VOID *pvBits;

        memoryBitmap = CreateDIBSection( memoryDC,
                                            &alpha,
                                            DIB_RGB_COLORS,
                                            &pvBits,
                                            NULL,
                                            0 );

        SelectObject(memoryDC, memoryBitmap);

        if ( (m_iLeftImageCrop+m_iRightImageCrop) >= m_iImageWidth || (m_iTopImageCrop+m_iBottomImageCrop) >= m_iImageHeight )
        {
            MessageBox(NULL, (m_strImagePath + "\nCheck your ImageCrop values!").c_str(), "ImageCrop Reset To 0!", MB_OK | MB_ICONWARNING | MB_TOPMOST | MB_SETFOREGROUND);
            m_iLeftImageCrop = m_iRightImageCrop = m_iTopImageCrop = m_iBottomImageCrop = 0;
        }

        m_iImageWidth = m_iImageWidth-m_iLeftImageCrop-m_iRightImageCrop;
        m_iImageHeight = m_iImageHeight-m_iTopImageCrop-m_iBottomImageCrop;
        
        if ( (m_iLeftImageEdge+m_iRightImageEdge) >= m_iImageWidth || (m_iTopImageEdge+m_iBottomImageEdge) >= m_iImageHeight )
        {
            MessageBox(NULL, (m_strImagePath + "\nCheck your ImageEdge values!").c_str(), "ImageEdge Reset To 0!", MB_OK | MB_ICONWARNING | MB_TOPMOST | MB_SETFOREGROUND);
            m_iLeftImageEdge = m_iRightImageEdge = m_iTopImageEdge = m_iBottomImageEdge = 0;
        }

        BitBlt(memoryDC, 0, 0, m_iImageWidth, m_iImageHeight, tmpmemoryDC, m_iLeftImageCrop, m_iTopImageCrop, SRCCOPY);
    
        tmpmemoryBitmap = (HBITMAP) SelectObject(tmpmemoryDC, oldtmpmemoryBMP);
        DeleteDC(tmpmemoryDC);  
        DeleteObject(tmpmemoryBitmap);
        DeleteObject(oldtmpmemoryBMP);

        PrepareImageEffects(memoryBitmap, bImageHasAlphaChannel, m_clrHueColor, m_iHueIntensity, m_iLuminanceIntensity, m_iSaturationIntensity, m_clrMixColor, m_iMixIntensity);
        
        if ( !m_bUseAlphaMap && bImageHasAlphaChannel )
            m_bUseAlphaMap = true;

        return true;
    }

    return false;
}

bool PaintTexture::Create_Icon()
{
    if(memoryBitmap)
    {
        memoryBitmap = (HBITMAP) SelectObject(memoryDC, memoryBitmap);
        DeleteObject(memoryBitmap);
    }
    if(memoryDC)
        DeleteDC(memoryDC);

    memoryBitmap = NULL;
    memoryDC = NULL;

    CIconExtractor m_IconExtractor;

    HICON noAlphaMaphIcon = NULL;
        
    int iMaxIconSize = m_iExtractIconSize;

    CStr strIconPath = m_strIconPath.c_str();
    strIconPath.Lower();
    
    if ( strIconPath.ReverseFind(".ico") == strIconPath.Length()-4 )
    {
        noAlphaMaphIcon = m_IconExtractor.ExtractIconFromIcoEx(strIconPath, memoryBitmap, iMaxIconSize);
    }
    else
    {
        int location = 0;
        if ( strIconPath.Find(",") != -1 && strIconPath.Length()-strIconPath.ReverseFind(",") <= 4 )
        {
            location = atoi( (strIconPath.Right(strIconPath.Length()-(strIconPath.ReverseFind(",")+1))));
            strIconPath = strIconPath.Left(strIconPath.ReverseFind(","));
        }
            
        if ( strIconPath.ReverseFind(".exe") == strIconPath.Length()-4 || strIconPath.ReverseFind(".dll") == strIconPath.Length()-4 )
        {
            noAlphaMaphIcon = m_IconExtractor.ExtractIconFromExeEx(strIconPath, location, memoryBitmap, iMaxIconSize);
        }
        else
        {
            SHFILEINFO shFileInfo;

            SHGetFileInfo(strIconPath, NULL, &shFileInfo, sizeof(shFileInfo), SHGFI_ICON | (m_iExtractIconSize < 32 ? SHGFI_SMALLICON : SHGFI_LARGEICON) );

            noAlphaMaphIcon = shFileInfo.hIcon;

            ICONINFO iconinfo;
            GetIconInfo(noAlphaMaphIcon, &iconinfo);

            GetLSBitmapSize(iconinfo.hbmColor, &m_iImageWidth, &m_iImageHeight);

            BITMAPINFO alpha;
            ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
            alpha.bmiHeader.biWidth=m_iImageWidth;      // Set size you need
            alpha.bmiHeader.biHeight=m_iImageHeight;    // Set size you need
            alpha.bmiHeader.biPlanes=1;
            alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
            alpha.bmiHeader.biSizeImage=0;
            alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
            alpha.bmiHeader.biClrUsed= 0;
            alpha.bmiHeader.biClrImportant= 0;
            VOID *pvBits;

            memoryDC = CreateCompatibleDC(NULL);

            memoryBitmap = CreateDIBSection( memoryDC,
                                                        &alpha,
                                                        DIB_RGB_COLORS,
                                                        &pvBits,
                                                        NULL,
                                                        0 );

            HBITMAP oldmemoryBitmap = (HBITMAP) SelectObject(memoryDC, memoryBitmap);

            HDC origDC = CreateCompatibleDC(NULL);
        
            HBITMAP oldorigBitmap = (HBITMAP) SelectObject(origDC, iconinfo.hbmColor);

            //Should be much quicker ;)
            BitBlt(memoryDC, 0, 0, m_iImageWidth, m_iImageHeight, origDC, 0, 0, SRCCOPY);
            
            /*int iy = 0;
            int ix = 0;
            
            COLORREF cl = 0;
        
            for(iy = 0; iy < m_iImageHeight; iy++)
            {
                for(ix = 0; ix < m_iImageWidth; ix++)
                {
                    cl = GetPixel(origDC, ix, iy);
                    
                    SetPixelV(memoryDC, ix, iy, cl);
                }
            }*/

            iconinfo.hbmColor = (HBITMAP) SelectObject(origDC, oldorigBitmap);
            DeleteDC(origDC);
            DeleteObject(oldorigBitmap);

            HDC darkDC = CreateCompatibleDC(NULL);

            HBITMAP darkBitmap = CreateDIBSection( darkDC,
                                                        &alpha,
                                                        DIB_RGB_COLORS,
                                                        &pvBits,
                                                        NULL,
                                                        0 );

            HBITMAP olddarkBitmap = (HBITMAP) SelectObject(darkDC, darkBitmap);

            HDC lightDC = CreateCompatibleDC(NULL);

            HBITMAP lightBitmap = CreateDIBSection( lightDC,
                                                        &alpha,
                                                        DIB_RGB_COLORS,
                                                        &pvBits,
                                                        NULL,
                                                        0 );

            HBITMAP oldlightBitmap = (HBITMAP) SelectObject(lightDC, lightBitmap);

            BITMAP bm_dark;
            GetObject(darkBitmap, sizeof(BITMAP), &bm_dark);

            BYTE *pPixel_dark = (LPBYTE)bm_dark.bmBits;

            int iSize = bm_dark.bmWidth * bm_dark.bmHeight;

            int j = 0;

            for (j = 0; j < iSize; j++)
            {
                pPixel_dark[0] = 0;
                pPixel_dark[1] = 0;
                pPixel_dark[2] = 0;
                pPixel_dark[3] = 255;

                pPixel_dark+= 4;
            }

            BITMAP bm_light;
            GetObject(lightBitmap, sizeof(BITMAP), &bm_light);

            BYTE *pPixel_light = (LPBYTE)bm_light.bmBits;

            for (j = 0; j < iSize; j++)
            {
                pPixel_light[0] = 255;
                pPixel_light[1] = 255;
                pPixel_light[2] = 255;
                pPixel_light[3] = 255;

                pPixel_light+= 4;
            }

            //-----------------------------------------------------------------------
            //Apply Icon with colorization
            //-----------------------------------------------------------------------
            DrawIconEx(darkDC, 0, 0, noAlphaMaphIcon, m_iImageWidth, m_iImageHeight, 0, NULL, DI_NORMAL);

            DrawIconEx(lightDC, 0, 0, noAlphaMaphIcon, m_iImageWidth, m_iImageHeight, 0, NULL, DI_NORMAL);
            //-----------------------------------------------------------------------

            GetObject(darkBitmap, sizeof(BITMAP), &bm_dark);
            pPixel_dark = (LPBYTE)bm_dark.bmBits;

            GetObject(lightBitmap, sizeof(BITMAP), &bm_light);
            pPixel_light = (LPBYTE)bm_light.bmBits;

            BITMAP bm_memory;
            GetObject(memoryBitmap, sizeof(BITMAP), &bm_memory);
            BYTE *pPixel_memory = (LPBYTE)bm_memory.bmBits;

            for (j = 0; j < iSize; j++)
            {
                if (pPixel_dark[0] == pPixel_light[0] && pPixel_dark[1] == pPixel_light[1] && pPixel_dark[2] == pPixel_light[2])
                    pPixel_memory[3] = 255;
                else if (pPixel_dark[0] == 0 && pPixel_dark[1] == 0 && pPixel_dark[2] == 0 && pPixel_light[0] == 255 && pPixel_light[1] == 255 && pPixel_light[2] == 255 )
                    pPixel_memory[3] = 0;
                else
                    pPixel_memory[3] = 255 - (pPixel_light[0] - pPixel_dark[0]);
            
                pPixel_dark+= 4;    
                pPixel_light+= 4;   
                pPixel_memory+= 4;
            }

            memoryBitmap = (HBITMAP) SelectObject(memoryDC, oldmemoryBitmap);
            DeleteDC(memoryDC);
            DeleteObject(oldmemoryBitmap);
            memoryDC = NULL;

            darkBitmap = (HBITMAP) SelectObject(darkDC, olddarkBitmap);
            DeleteDC(darkDC);
            DeleteObject(darkBitmap);
            DeleteObject(olddarkBitmap);

            lightBitmap = (HBITMAP) SelectObject(lightDC, oldlightBitmap);
            DeleteDC(lightDC);
            DeleteObject(lightBitmap);
            DeleteObject(oldlightBitmap);

            DeleteObject(iconinfo.hbmColor);
            DeleteObject(iconinfo.hbmMask);

            DestroyIcon(noAlphaMaphIcon);
            noAlphaMaphIcon = NULL;
        }
    }

    if ( memoryBitmap != NULL )
    {
        GetLSBitmapSize(memoryBitmap, &m_iImageWidth, &m_iImageHeight);
    }
    else
    {
        m_iImageWidth = iMaxIconSize;
        m_iImageHeight = iMaxIconSize;
    }

    if ( memoryBitmap != NULL || noAlphaMaphIcon != NULL )
        PrepareIconEffects(memoryDC, memoryBitmap, m_clrHueColor, m_iHueIntensity, m_iLuminanceIntensity, m_iSaturationIntensity, m_clrMixColor, m_iMixIntensity, m_iImageWidth, noAlphaMaphIcon);
    
    if (noAlphaMaphIcon)
        DestroyIcon(noAlphaMaphIcon);

    return true;
}
